@@ -12,7 +12,9 @@
 #include "config.h"
 
 /* this is a work-around until we manage to fix configure.ac */
+#ifndef _AIX
 #pragma GCC diagnostic ignored "-Wdeclaration-after-statement"
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -89,11 +91,15 @@ get_string_component(struct fjson_object *jso)
  * extra table lookup costs performance, but right the contrary
  * is the case. We get amore than 30% performance increase due
  * to it (compared to the latest version of the code that did not
- * do the lookups.
+ * do the lookups).
  * rgerhards@adiscon.com, 2015-11-18
+ * I have renamed needsEscape to char_needsEscape and made it
+ * external. This makes it possible to share the implementation
+ * with the newly contributed json_print.c module.
+ * rgerhards, 2016-11-30
  */
 
-static char needsEscape[256] = {
+char char_needsEscape[256] = {
 	1, 1, 1, 1, 1, 1, 1, 1, /* ascii codes 0 .. 7 */
 	1, 1, 1, 1, 1, 1, 1, 1,
 	1, 1, 1, 1, 1, 1, 1, 1,
@@ -132,7 +138,7 @@ static void fjson_escape_str(struct printbuf *pb, const char *str)
 {
 	const char *start_offset = str;
 	while(1) { /* broken below on 0-byte */
-		if(needsEscape[*((unsigned char*)str)]) {
+		if(char_needsEscape[*((unsigned char*)str)]) {
 			if(*str == '\0')
 				break;
 			if(str != start_offset)
@@ -195,9 +201,11 @@ int fjson_object_put(struct fjson_object *jso)
 
 static void fjson_object_generic_delete(struct fjson_object* jso)
 {
-	printbuf_free(jso->_pb);
-	DESTROY_ATOMIC_HELPER_MUT(jso->_mut_ref_count);
-	free(jso);
+	if (jso) {
+		printbuf_free(jso->_pb);
+		DESTROY_ATOMIC_HELPER_MUT(jso->_mut_ref_count);
+		free(jso);
+	}
 }
 
 static struct fjson_object* fjson_object_new(const enum fjson_type o_type)
@@ -714,6 +722,7 @@ static int fjson_object_double_to_json_string(struct fjson_object* jso,
 {
 	char buf[128], *p, *q;
 	int size;
+	double dummy;  /* needed for modf() */
 	/* Although JSON RFC does not support
 	 * NaN or Infinity as numeric values
 	 * ECMA 262 section 9.8.1 defines
@@ -727,7 +736,9 @@ static int fjson_object_double_to_json_string(struct fjson_object* jso,
 		else
 			size = snprintf(buf, sizeof(buf), "-Infinity");
 	else
-		size = snprintf(buf, sizeof(buf), "%.17g", jso->o.c_double);
+		size = snprintf(buf, sizeof(buf),
+			(modf(jso->o.c_double, &dummy)==0)?"%.17g.0":"%.17g",
+			jso->o.c_double);
 
 	p = strchr(buf, ',');
 	if (p) {
